@@ -97,7 +97,6 @@ async def get_item(client: TonlibClient, addr: str, nft_items: list, items: dict
             content = await get(content_url)
     else:
         content = await get(content_url)
-
     collection_content = await get_collection_content(client, x['collection_address'].to_string())
     result = {
         'address': nft_address,
@@ -148,26 +147,26 @@ async def _get_collection(client: TonlibClient, addr: str):
     return result
 
 
-def get_all_wallet_transactions_raw(addr: str, client: TonlibClient = None):
+def get_all_wallet_transactions_raw(addr: str, limit: int = 10**9, client: TonlibClient = None):
     """
     highly recommend to use with client which ls index = 2, for e.g. client = get_client(ls=2)
     """
     if not client:
         client = get_client(2)
-    return asyncio.get_event_loop().run_until_complete(_get_all_wallet_transactions_raw(client, addr))
+    return asyncio.get_event_loop().run_until_complete(_get_all_wallet_transactions_raw(client, addr, limit))
 
 
-def get_all_wallet_transactions(addr: str, client: TonlibClient = None):
+def get_all_wallet_transactions(addr: str, limit: int = 10**9, client: TonlibClient = None):
     """
     highly recommend to use with client which ls index = 2, for e.g. client = get_client(ls=2)
     """
     if not client:
         client = get_client(2)
-    return asyncio.get_event_loop().run_until_complete(_get_all_wallet_transactions(client, addr))
+    return asyncio.get_event_loop().run_until_complete(_get_all_wallet_transactions(client, addr, limit))
 
 
-async def _get_all_wallet_transactions(client: TonlibClient, addr: str):
-    all_transactions = await _get_all_wallet_transactions_raw(client, addr)
+async def _get_all_wallet_transactions(client: TonlibClient, addr: str, limit: int = 10**9):
+    all_transactions = await _get_all_wallet_transactions_raw(client, addr, limit)
     result = []
     for tr in all_transactions:
         tr_result = {
@@ -196,20 +195,20 @@ async def _get_all_wallet_transactions(client: TonlibClient, addr: str):
     return result
 
 
-async def _get_all_wallet_transactions_raw(client: TonlibClient, addr: str):
+async def _get_all_wallet_transactions_raw(client: TonlibClient, addr: str, limit: int = 10**9):
     account = await client.find_account(addr)
     all_transactions = []
-    limit = 30  # must be more than 1
-    trs = await account.get_transactions(limit=limit)
+    lim = 30  # must be more than 1
+    trs = await account.get_transactions(limit=lim)
     for tr in trs:
         all_transactions.append(tr.to_json())
-    while len(trs) == limit:
+    while len(trs) == lim and len(all_transactions) < limit:
         trs = await account.get_transactions(from_transaction_lt=trs[-1].to_json()['transaction_id']['lt'],
                                              from_transaction_hash=trs[-1].to_json()['transaction_id']['hash'],
-                                             limit=limit)
+                                             limit=lim)
         for tr in trs[1:]:
             all_transactions.append(tr.to_json())
-    return all_transactions
+    return all_transactions[:limit]
 
 
 def get_nft_items_by_owner_address(addr: str, client: TonlibClient = None):
@@ -224,17 +223,51 @@ def get_nft_items_by_owner_address(addr: str, client: TonlibClient = None):
 async def _get_nft_items_by_owner_address(client: TonlibClient, addr: str):
     result = set()
     all_transactions = await _get_all_wallet_transactions(client, addr)
-    for tr in all_transactions:
+    await client.tonlib_wrapper.close()
+    client = await _get_client(client.ls_index)
+
+    async def proc_tr(tr):
+
         if tr['type'] == 'in':
+            if 'EQBdHpUa1u3IVs1b3xfjpSlXVD5lkNsfIcJMqtXu1IJ0rLHK' == tr['from'] or 'EQBqkkKeF7cwbX-erNK73qs4fqpwkXNQd_yqS4HXzp3PaCSX' == tr['from']:
+                print(tr)
             account = await client.find_account(tr['from'])
             try:
                 nft_data = await account.get_nft_data()
                 owner = nft_data['owner_address'].to_string()
-                sale = await get_nft_sale(client, addr)
+                sale = await get_nft_sale(client, owner)
                 if sale:
                     owner = sale['owner']['address']
                 if owner == Address(addr).to_string(is_user_friendly=False):
                     result.add(tr['from'])
             except:
-                continue
+                return
+
+    await asyncio.gather(*[proc_tr(tr) for tr in all_transactions])
     return list(result)
+
+
+def get_collection_items(addr: str, client: TonlibClient = None):
+    if not client:
+        client = get_client()
+    return asyncio.get_event_loop().run_until_complete(_get_collection_items(client, addr))
+
+
+async def _get_collection_items(client: TonlibClient, addr: str):
+    account = await client.find_account(addr)
+    items_amount = await account.get_collection_data()
+    items_amount = items_amount['next_item_index']
+
+    async def get_item_addr(index, result):
+        item = await account.get_nft_address_by_index(index=index)
+        # result.append(item['address'].to_string(True, True, True)) # human-readable bounceable form
+        result.append(item['address'].to_string()) # raw form
+
+    result = []
+
+    for p in range(ceil(items_amount / 10000)):
+        await asyncio.gather(*[get_item_addr(i, result) for i in range(p * 10000, min(items_amount, 10000 * (p + 1)))])
+
+    if len(result) < items_amount:
+        raise Exception('GOT NOT ALL ITEMS')
+    return result
